/** 
 *  \file   	gui.h
 *	\brief  	Definitions for DreamShell GUI
 *	\date		2004-2011
 *	\author		SWAT
 *	\copyright	http://www.dc-swat.ru
 */

#ifndef _DS_GUI_H
#define _DS_GUI_H


#include "video.h"
#include "list.h"
#include "events.h"
#include "SDL/SDL_gui.h"



typedef struct MouseCursor {
    
    SDL_Surface *bg;
    SDL_Surface *cursor;
    SDL_Rect src, dst;
    int draw;

} MouseCursor_t;


/*
  Create cursor from file or surface.
*/
MouseCursor_t *CreateMouseCursor(const char *fn, /* or */ SDL_Surface *surface);
void DestroyMouseCursor(MouseCursor_t *c);
void DrawMouseCursor(MouseCursor_t *c);
void SetActiveMouseCursor(MouseCursor_t *c);
MouseCursor_t *GetActiveMouseCursor();
void UpdateActiveMouseCursor();
void DrawActiveMouseCursor();



#define VIRT_KB_SURFACE_COUNT 3

#define VIRT_KB_SURFACE_KEYS 0
#define VIRT_KB_SURFACE_KEYS_CAPS 1
#define VIRT_KB_SURFACE_NUMS 2



typedef struct virt_kb_section {
	
  int keys[4];
  
} virt_kb_section_t;


typedef struct virt_kb_syms {
	
	virt_kb_section_t sections[9];
	
} virt_kb_syms_t;


typedef struct virt_kb_surface {
	
	SDL_Surface *s;
	float zoom;
	
} virt_kb_surface_t;



typedef struct virt_kb {
	
	int cur_surf;
	int visible;
	int redraw;
	
	int row;
	int col;
	
	SDL_Event event;
	
	uint32 clr;
	
	virt_kb_surface_t *surface;
	
	Event_t *input;
	Event_t *video;
	
	//Sem_t *sem;
	
} virt_kb_t;



int VirtKeyboardInit();
void VirtKeyboardShutdown();
int VirtKeyboardVisible();
void VirtKeyboardReDraw();
void VirtKeyboardShow();
void VirtKeyboardHide();
void VirtKeyboardToggle();
virt_kb_surface_t *VirtKeyboardGetSurface();
void VirtKeyboardSetSurface(int index);

int InitGUI();
void ShutdownGUI();
int LockGUI();
int UnLockGUI();
int GUI_IsLocked();
int GUI_Object2Trash(GUI_Object *object);
void GUI_ClearTrash();

Uint32 colorHexToRGB(char *color, SDL_Color *clr);
SDL_Color Uint32ToColor(Uint32 c);
Uint32 ColorToUint32(SDL_Color c);
Uint32 MapHexColor(char *color, SDL_PixelFormat *format);

SDL_Surface *SDL_ImageLoad(const char *filename, SDL_Rect *selection);

#endif
