/** 
 *  \file   	utils.h
 *	\brief  	Definitions for DreamShell utils
 *	\date		2004-2011
 *	\author		SWAT
 *	\copyright	http://www.dc-swat.ru
 */


#ifndef _DS_UTILS_H
#define _DS_UTILS_H


/* Macros v2 for accessing/creating DreamShell version codes */
	
#define DS_VER_MAJOR(v) ((v >> 24) & 0xff)
#define DS_VER_MINOR(v) ((v >> 16) & 0xff)
#define DS_VER_MICRO(v) ((v >> 8) & 0xff)
#define DS_VER_BUILD(v) ((v >> 0) & 0xff)

/* Divide build to type and number (if using this build structure) */
#define DS_VER_BUILD_TYPE(b) 		(b & 0xf0) / 0xf)
#define DS_VER_BUILD_TYPE_STR(b) 	(GetVersionBuildTypeString(DS_VER_BUILD_TYPE(b))
#define DS_VER_BUILD_NUM(b) 		(b & 0x0f)

#define DS_MAKE_VER(MAJOR, MINOR, MICRO, BUILD) \
					( (((MAJOR) & 0xff) << 24) 	\
					| (((MINOR) & 0xff) << 16) 	\
					| (((MICRO) & 0xff) << 8) 	\
					| (((BUILD) & 0xff) << 0) )


void readstr(FILE *f, char *string);
int FileSize(const char *fn);
int FileExists(const char *fn);
int DirExists(const char *dir);
int PeriphExists(const char *name);

// Deprecated. Using realpath.
int makeabspath(char *buff, char *path, size_t size);

char *realpath(const char *path, char resolved[PATH_MAX]);

int makeabspath_wd(char *buff, char *path, char *dir, size_t size);
const char *relativeFilePath(const char *rel, const char *file);
int relativeFilePath_wb(char *buff, const char *rel, const char *file);
char *getFilePath(const char *file);

// Deprecated. Use chdir or fs_chdir
int ds_chdir(char *dir);


#ifdef DS_ARCH_DC
void arch_shutdown();
#endif

void ds_sleep(int ms);

int splitline(char **argv, int max, char *line);

int hex_to_int(char c);
char *strtolower(char *str);

#if !defined(DS_ARCH_DC) || !defined(DS_ARCH_PSP)
char *strsep(char **stringp, const char *delim);
#endif

char* substring(const char* str, size_t begin, size_t len);

#endif
